﻿
; Основная функция работы с LPT портом. Должна регулярно вызываться 10 раз в секунду.
Procedure LPT_Core()
  Static CountErrOut, LedPulse, PulseCount
  Protected OutBuff.u = 0, ErrStatus=#False
  
  If dPort::dpID()=0 ; Драйвер не активен.
    CountErrOut+1
    If CountErrOut>=100
      CountErrOut=0
      LogWrite("Ошибка записи данных в LPT порт.", #Log_TM_Warning)
    EndIf
    ProcedureReturn
  EndIf
    
  ; Пульсирующий сигнал.
  If PulseCount>=45
    SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Pulse]))
    If LedPulse<>#True
      LedPulse=#True
      SetLedColor(#Led_Pulse, #LedColor_Green)
    EndIf
  Else
    If LedPulse<>#False
      LedPulse=#False
      SetLedColor(#Led_Pulse, #LedColor_Off)
    EndIf
  EndIf
  
  PulseCount+1
  If PulseCount>=50
    PulseCount=0
  EndIf
    
  If gVar\Logic\Alarm=0 ; Аварийные датчики не сработали.
    
    With gVar\Logic\Col1
      If \Status = #StatusStop Or \Status=#StatusPause ; Ручное управление клапанами.
        If \ControlValves>0
          Select \ControlValves
            Case 1 ; Первый клапан.
              SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Valve1]))
            Case 2 ; Второй клапан.
              SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Valve2]))
            Case 3 ; Третий клапан.
              SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Valve3]))
          EndSelect
        EndIf
      ElseIf \Status = #StatusStart ; Автоматическое управление клапанами.
        If \Mode=#Col1_Mode_Valve1 Or \Mode=#Col1_Mode_Valve2 Or \Mode=#Col1_Mode_Valve3
          
            \CounterPWM+1 ; ШИМ.
            If \CounterPWM>=100
              \CounterPWM=0
            EndIf
            
            If \PWM > \CounterPWM
              Select \Mode
                Case #Col1_Mode_Valve1
                  SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Valve1]))
                Case #Col1_Mode_Valve2
                  If \v2_CloseState=#False
                    SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Valve2]))
                  EndIf
                Case #Col1_Mode_Valve3
                  SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Valve3]))
              EndSelect
            EndIf
          
        EndIf
      EndIf
    
      If \Status <> #StatusStop
        SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Col1]))
      EndIf
    
    EndWith
    
    
    If gVar\Logic\Col2\Work
      SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Col2]))
    EndIf
    
  Else
    ErrStatus=#True
  EndIf
  
  
  If gVar\Logic\Col3\Work
    If gVar\Logic\Col3\PinMax
      SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Col3_Max]))
    EndIf
    
    If gVar\Logic\Col3\PinMin
      SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Col3_Min]))
    EndIf
  EndIf
  
  If gSetting\Logic\Check_d6 And ; Разрешена работа датчика 6
     gVar\Logic\d6_Pin=#True      ; и нужно включить канал.
    SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_d6]))
  EndIf
  
  
  If ErrStatus=#False
    With gVar\Sensors
      ForEach \Sensor()
        If \Sensor()\Use>#Sensor_Use_None And TestBit(\Sensor()\Flag, #SensorsFlag_MoreErr)
          ErrStatus=#True
          Break
        EndIf
      Next
    EndWith
  EndIf
  
  If ErrStatus=#True
    SetBit(OutBuff, NumToBit(gSetting\LPT\Pins[#LPT_Pin_Err]))
  EndIf
  
  OutBuff ! %0000101100000000 ; Инвертирование данных для выходов Strobe, AutoLF и SelectIn.
  OutBuff & %0000111111111111
  
  If dPort::dpOut(gSetting\LPT\Adr+2, (OutBuff>>8)&$FF)
    dPort::dpOut(gSetting\LPT\Adr, OutBuff&$FF)
    CountErrOut=0
  Else
    CountErrOut+1
    If CountErrOut>=100
      CountErrOut=0
      LogWrite("Ошибка записи данных в LPT порт.", #Log_TM_Warning)
    EndIf
  EndIf
EndProcedure

; IDE Options = PureBasic 5.31 (Windows - x86)
; Folding = -
; EnableUnicode
; EnableXP