﻿
#Sensor_Port = 2 ; Идентификатор COM порта, к которому подключены датчики.

Procedure FindSensor_Use(Use.a) ; Возвращает указатель на структуру требуемого датчика. Только для основного потока!
  Protected *Point=0
  
  ForEach gVar\Sensors\Sensor()
    If Sensor_IsTermoSensor(@gVar\Sensors\Sensor())
      If gVar\Sensors\Sensor()\Use = Use
        *Point=@gVar\Sensors\Sensor()
        Break
      EndIf
    EndIf
  Next
  
  ProcedureReturn *Point
EndProcedure

Procedure Sensor_IsTermoSensor(*Sensor.SensorParam)
  Protected Result=#False
  
  If *Sensor And (*Sensor\SN\Type = $10 Or *Sensor\SN\Type = $28) ; Это датчик температуры.
    Result = #True
  EndIf
  
  ProcedureReturn Result
EndProcedure

;- Процедуры ниже для отдельного потока, работы с 1-Wire.

Procedure Sensor_Find(List s.OW_SN()) ; Поиск 1-Wire устройств.
  Protected Sensor.OW_SN, Mode
  
  If ListSize(s())>0
    ClearList(s())
  EndIf
  
  Mode = OW_GetMode()
  OW_SetMode(ClearBit(Mode, #OW_Mode_CPU_MinUsage)) ; Для более быстрого поиска.
  
  OW_SearchROM_Reset()
  
  While OW_SearchROM_Enum(@Sensor, #Sensor_Port)=#True
    If PeekQ(@Sensor)<>0 And OW_GetCRC(@Sensor, 7) = Sensor\CRC
      If AddElement(s())
        CopyStructure(@Sensor, @s(), OW_SN)
      EndIf
    EndIf
  Wend  
  
  OW_SetMode(Mode)
  
EndProcedure


Procedure sTh_Find(*Sensors.ThS_Sensors, *Param.ThSParam)
  Protected NewList Sensors.OW_SN(), x, Result = #False
  
  Sensor_Find(Sensors())
  
  x = ListSize(Sensors())
  LogPost("Найдено "+x+"  '1-Wire' "+wordform(x, "устройство", "устройства", "устройств"), #Log_TM_Info)
  
  ForEach *Sensors\Sensor()
    x=#False
    ForEach Sensors()
      If *Sensors\Sensor()\ID = PeekQ(@Sensors())
        x=#True
        DeleteElement(Sensors())
      EndIf
    Next
    
    If x=#True
      ClearBit(*Sensors\Sensor()\Flag, #SensorsFlag_NotFound)
    Else
      SetBit(*Sensors\Sensor()\Flag, #SensorsFlag_NotFound)
      Result = #True
    EndIf
    
    If Sensor_IsTermoSensor(@*Sensors\Sensor()\SN)
      SetBit(*Sensors\Sensor()\Flag, #SensorsFlag_TermoSensor)
    Else
      ClearBit(*Sensors\Sensor()\Flag, #SensorsFlag_TermoSensor)
    EndIf
  Next
  
  ClearList(*Param\Sensor())
  
  If ListSize(Sensors())>0
    LastElement(*Sensors\Sensor())
    ForEach Sensors()
      If AddElement(*Sensors\Sensor())
        *Sensors\Sensor()\ID = PeekQ(@Sensors())
        *Sensors\Sensor()\Use = #Sensor_Use_None
        *Sensors\Sensor()\Flag = 0
        If Sensor_IsTermoSensor(@*Sensors\Sensor()\SN)
          SetBit(*Sensors\Sensor()\Flag, #SensorsFlag_TermoSensor)
        Else
          ClearBit(*Sensors\Sensor()\Flag, #SensorsFlag_TermoSensor)
        EndIf
        Result = #True
      EndIf
      
      If AddElement(*Param\Sensor())
        *Param\Sensor()\ID=PeekQ(@Sensors())
      EndIf
    Next
  EndIf
  
    
  ProcedureReturn Result
EndProcedure


Procedure Sensor_OpenPort(*Sensors.ThS_Sensors, NumPort.a) ; Открытие COM порта, к которому подключены датчики.
  Protected Result=0
  
  OW_ClosePort(#Sensor_Port)
  If NumPort>0
    Result=OW_OpenPort("COM"+NumPort, #Sensor_Port)
  EndIf
  
  If *Sensors
    With *Sensors
      SetBit(\Flags, #SensorsAll_Flag_Start)
      \CurrentSensor=0
      ForEach \Sensor()
        ClearBit(\Sensor()\Flag, #SensorsFlag_Err|#SensorsFlag_MoreErr|
                                 #SensorsFlag_Update|#SensorsFlag_ConvertT)
        \Sensor()\CountErr=0
        \Sensor()\Count85Err=0
        \Sensor()\CountNoUpdate=0
      Next
    EndWith
  EndIf
  
  ProcedureReturn Result
EndProcedure


Procedure Sensor_ConvertT(*Sensors.ThS_Sensors, All=#False, UseAll=#False)
  Shared EndProg
  Protected i=0
    
  If *Sensors\CurrentSensor>=ListSize(*Sensors\Sensor())
    *Sensors\CurrentSensor=0
  EndIf
  
  ForEach *Sensors\Sensor()
    With *Sensors\Sensor()
      If All Or i>=*Sensors\CurrentSensor
        If (UseAll Or \Use>#Sensor_Use_None) And 
           Sensor_IsTermoSensor(@*Sensors\Sensor()) ; Это датчик температуры.
          
          ClearBit(\Flag, #SensorsFlag_Err)
          
          If OW_SelectROM(@\SN, #Sensor_Port)  ; Выбираем датчик
            If OW_ConvertT(0, #Sensor_Port)    ; и запускаем измерение температуры.
              SetBit(\Flag, #SensorsFlag_ConvertT)
            Else
              SetBit(\Flag, #SensorsFlag_Err)
            EndIf
          Else
            SetBit(\Flag, #SensorsFlag_Err)
          EndIf
                    
          If All=#False
            Break
          EndIf
          
        EndIf
      EndIf
    EndWith
    i+1
    
    If EndProg=#True
      Break
    EndIf
  Next 
  
  *Sensors\CurrentSensor=i
EndProcedure


Procedure Sensor_GetTermo(*Sensors.ThS_Sensors, All=#False, UseAll=#False)
  Shared EndProg
  Protected Buff.OW_RAM, i, Termo.f, t.w, x, Count
  
  ForEach *Sensors\Sensor()
    With *Sensors\Sensor()
      If (All Or *Sensors\CurrentSensor=i) And (UseAll Or \Use>#Sensor_Use_None)
        If Sensor_IsTermoSensor(@*Sensors\Sensor()) ; Это датчик температуры.
           
          
          If TestBit(\Flag, #SensorsFlag_Err) = #False And
             TestBit(\Flag, #SensorsFlag_ConvertT)
            
            If OW_SelectROM(@\SN, #Sensor_Port)        ; Выбираем датчик
              If OW_ReadRAM(@Buff, #Sensor_Port)       ; и считываем данные температуры.
                If OW_GetCRC(@Buff, 8)=Buff\CRC
                  Termo = PeekW(@Buff\L_Termo)
                  If \SN\Type = $28 ; DS18B20.
                    Termo/16
                  Else              ; DS1820 или DS18S20.
                    Termo/2 
                  EndIf
                  
                  t=Termo*10
                  If t<>850
                    \Termo = t
                    \CountErr = 0
                    \Count85Err=0
                    ClearBit(\Flag, #SensorsFlag_MoreErr)
                    SetBit(\Flag, #SensorsFlag_Update)
                  Else
                    x=#Sensor_Max85Err
                    If *Sensors\Serial
                      Count = ListSize(*Sensors\Sensor())
                      If Count>0
                        If Count>6
                          Count=6
                        EndIf
                        x=Round(x/Count, #PB_Round_Up)
                      EndIf
                    EndIf
                    If \Count85Err>x
                      SetBit(\Flag, #SensorsFlag_MoreErr)
                    Else
                      \Count85Err+1
                    EndIf
                  EndIf
                  
                Else
                  SetBit(\Flag, #SensorsFlag_Err)
                EndIf
              Else
                SetBit(\Flag, #SensorsFlag_Err)
              EndIf
            Else
              SetBit(\Flag, #SensorsFlag_Err)
            EndIf
          EndIf
          
          If TestBit(\Flag, #SensorsFlag_Err)
            x=#Sensor_MaxCountErr
            If *Sensors\Serial
              Count = ListSize(*Sensors\Sensor())
              If Count>0
                If Count>6
                  Count=6
                EndIf
                x=Round(x/Count, #PB_Round_Up)
              EndIf
            EndIf
            If \CountErr<=x
              \CountErr+1
            Else
              SetBit(\Flag, #SensorsFlag_MoreErr)
            EndIf
          EndIf
          
          If All=#False
            *Sensors\CurrentSensor+1
            Break
          EndIf
          
          ClearBit(\Flag, #SensorsFlag_ConvertT)
        EndIf
      EndIf
    EndWith
    i+1
    
    If EndProg=#True
      Break
    EndIf
    
  Next 
EndProcedure

Procedure Sensor_Core(*Sensors.ThS_Sensors)
  Shared EndProg
  Protected Result = #False
  
  With *Sensors
    If \PortID
      
      If TestBit(\Flags, #SensorsAll_Flag_Start)=#False
        Sensor_GetTermo(*Sensors, Bool(*Sensors\Serial=0), *Sensors\IsWindow)
      Else
        ClearBit(\Flags, #SensorsAll_Flag_Start)
      EndIf
      
      If EndProg=#False
        ; Запуск измерения температуры.
        Sensor_ConvertT(*Sensors, Bool(*Sensors\Serial=0), *Sensors\IsWindow)
        Result = #True
      EndIf
      
    EndIf
  EndWith
  
  ProcedureReturn Result
EndProcedure

Procedure ThS_ModifySetting(*Sensors.ThS_Sensors, Flags.a, *Param.ThSParam)
  Protected x
  
  If TestBit(Flags, #ThS_FlagIn_ModifyPort)
    *Sensors\PortID=Sensor_OpenPort(*Sensors, *Sensors\NumPort)
    If *Sensors\PortID=0 And *Sensors\NumPort>0
      LogPost("Не удалось открыть порт COM"+*Sensors\NumPort, #Log_TM_Error)
    EndIf
  EndIf
  
  If *Param
    If TestBit(Flags, #ThS_FlagIn_Setting)
      ForEach *Sensors\Sensor()
        
        x=#False
        ForEach *Param\Sensor()
          If *Sensors\Sensor()\ID = *Param\Sensor()\ID
            x=#True
            *Sensors\Sensor()\Use =  *Param\Sensor()\Use
            Break
          EndIf
        Next
        
        If x=#False
          DeleteElement(*Sensors\Sensor())
        EndIf
        
      Next
    EndIf
  EndIf
  
EndProcedure

Procedure sTh_DataIO(*Param.ThSParam, *Sensors.ThS_Sensors)
  
  
  LockMutex(*Param\Mutex)
  
  If TestBit(*Param\Flags, #ThS_FlagIn_Find)
    ClearBit(*Param\Flags, #ThS_FlagIn_Find)
    If sTh_Find(*Sensors, *Param.ThSParam)
      SetBit(*Param\Flags, #ThS_FlagOut_Find)
    EndIf
  EndIf
  
  
  
  If TestBit(*Param\Flags, #ThS_FlagIn_ModifyPort) Or TestBit(*Param\Flags, #ThS_FlagIn_Setting)
    *Sensors\Serial = *Param\Serial
    *Sensors\NumPort = *Param\NumPort
    ThS_ModifySetting(*Sensors, *Param\Flags, *Param)
    ClearBit(*Param\Flags, #ThS_FlagIn_ModifyPort|#ThS_FlagIn_Setting)
  EndIf
  
  If TestBit(*Param\Flags, #ThS_FlagIn_WinSetting)
    *Sensors\IsWindow = *Param\IsWindow
    ClearBit(*Param\Flags, #ThS_FlagIn_WinSetting)
  EndIf
  
  If TestBit(*Param\Flags, #ThS_FlagOut_Find)=0
    *Param\PortID = *Sensors\PortID
    ClearList(*Param\Sensor())
    If CopyList(*Sensors\Sensor(), *Param\Sensor())
      SetBit(*Param\Flags, #ThS_FlagOut_SetData)
    EndIf
  EndIf

  UnlockMutex(*Param\Mutex)
  
  PostEvent(#Event_SensorData, #Window_Main, 0)
EndProcedure

Procedure Sensor_Thread(*Param.ThSParam)
  Shared EndProg
  Protected Sensors.ThS_Sensors, p.ThSParam
  Protected i
  
  If *Param
    
    LockMutex(*Param\Mutex)
    Sensors\NumPort=*Param\NumPort
    Sensors\Serial=*Param\Serial
    CopyList(*Param\Sensor(), Sensors\Sensor())
    UnlockMutex(*Param\Mutex)
    
    ThS_ModifySetting(@Sensors, #ThS_FlagIn_ModifyPort, 0)
    
    Repeat
      
      Sensor_Core(@Sensors)
      sTh_DataIO(*Param, @Sensors)
      
      ForEach Sensors\Sensor()
        ClearBit(Sensors\Sensor()\Flag, #SensorsFlag_Update)
      Next
      
      For i=1 To 18
        If EndProg=#True
          OW_ClosePort(#Sensor_Port)
          Sensors\PortID=0
          Break 2
        EndIf
        Delay(50)
      Next i
      
    ForEver
    
  EndIf
  
EndProcedure


; IDE Options = PureBasic 5.31 (Windows - x86)
; Folding = --
; EnableUnicode
; EnableXP